<?php
if ( ! defined( 'ABSPATH' ) ) {
    exit; // Exit if accessed directly
}
add_action('admin_menu', 'doshii_settings_setup');
function doshii_settings_setup() {
	add_options_page('Doshii Settings', 'Doshii Settings', 'manage_options', 'doshii-settings', 'doshii_admin_options_page');
}

function doshii_admin_options_page() {
  global $doshii_options, $doshii_api_key, $wpdb;

  // check user capabilities
  if ( ! current_user_can( 'manage_options' ) ) {
    return;
  }

  //Get the active tab from the $_GET param
  $default_tab = null;
  $tab = isset($_GET['tab']) ? $_GET['tab'] : $default_tab;

  $token = $doshii_api_key['token'];
  ?>

  <div class="wrap">
    <?php printf('<img src="%1$s" alt="Doshii" style="width:100%%;"/>', plugins_url( '/assets/images/doshii_logo.png', __FILE__ )); ?>
    <h1><?php echo esc_html( get_admin_page_title() ); ?></h1>

    <nav class="nav-tab-wrapper">
      <a href="?page=doshii-settings" class="nav-tab <?php if($tab===null):?>nav-tab-active<?php endif; ?>">Settings</a>
      <a href="?page=doshii-settings&tab=initialisation" class="nav-tab <?php if($tab==='initialisation'):?>nav-tab-active<?php endif; ?>">Initialisation</a>
      <a href="?page=doshii-settings&tab=menu" class="nav-tab <?php if($tab==='menu'):?>nav-tab-active<?php endif; ?>">Menu</a>
    </nav>

    <div class="tab-content">
        <?php switch($tab) :
          case 'initialisation':
            $message = '* Doshii Integration already initialised.';
            if (empty($token)) {
                $message = '* Doshii initialisation has not been performed.';
            }
            $locationId = $doshii_options['location_id'];
            if (empty($locationId)) {
                $message = 'Doshii location ID has not been configured on the Settings tab.';
            } else {
                $table  = $wpdb->prefix . 'doshii_menu';
                $doshii_menu_sql = "SELECT doshii_monitor_stock, doshii_filtered_menu FROM $table WHERE location_id = '$locationId'";
                $doshiiResults = $wpdb->get_row($doshii_menu_sql) or array();
                $monitorStock = empty($doshiiResults->doshii_monitor_stock) ? 'no' : $doshiiResults->doshii_monitor_stock;
                $isMonitoring = empty($doshii_options['stock_availability']) ? 'no' : $doshii_options['stock_availability'];
                if ($isMonitoring !== $monitorStock) {
                    $message = '* Re-initialisation is required as Stock Availability monitoring has changed.';
                } else if ($isMonitoring === 'yes') {
                    $filtered = empty($doshiiResults->doshii_filtered_menu) ? 'no' : $doshiiResults->doshii_filtered_menu;
                    $nowFiltered = empty($doshii_options['use_menu_management']) ? 'no' : $doshii_options['use_menu_management'];
                    if ($filtered !== $nowFiltered) {
                        $message = '* Re-initialisation is required as Menu Management setting has changed.';
                    }
                }
            }

            ?>
                <h2>
                    <?php _e('Initialise Doshii', 'doshii'); ?>
                </h2>
                <table class="form-table">
                    <tbody>
                        <tr valign="top">
                            <th scope="row" valign="top">
                                <?php _e('Location Token *', 'doshii'); ?>
                            </th>
                            <td>
                                <input type="password" id="locationToken" class="regular-text">
                            </td>
                        </tr>
            <?php
                if (!empty($message)) {
            ?>
                        <tr valign="top">
                            <th scope="row" valign="top"></th>
                            <td><?php echo $message ?></td>
                        </tr>
            <?php
                }
            ?>
                        <tr valign="top">
                            <th scope="row" valign="top"></th>
                            <td>
                                <div>
                                    <input  class="button-primary btn" type="button" value="Initialise" id="initialiseBtn" name="" onclick="doshii_initialise()">
                                </div>
                                <?php printf(
                                    '<img src="%1$s" alt="progress spinner" style="display:none" id="initialising"/>',
                                    plugins_url( '/assets/images/loading.gif', __FILE__ )
                                ); ?>
                                <p class="message"></p>
                            </td>
                        </tr>
                    </tbody>
                </table>
                <script>
                function doshii_initialise() {
                    jQuery('#initialiseBtn').attr('disabled','disabled');
                    jQuery('#initialising').show();
                    jQuery.ajax({
                     type : "GET",
                     dataType : "JSON",
                     url : '<?php echo admin_url( 'admin-ajax.php' ); ?>',
                     data : {
                        action: "doshii_initialise_with_woocommerce",
                        key: jQuery('#locationToken').val()
                     },
                     success: function(response) {
                        jQuery('.message').html(response['message']);
                        jQuery('#initialiseBtn').removeAttr('disabled','disabled');
                        jQuery('#initialising').hide();
                     },
                     error: function(request,status,errorThrown) {
                        console.log(request);
                        console.log(status);
                        console.log(errorThrown);
                        jQuery('.menuUpdated').html('');
                        jQuery('.message').html('Failed to synchronise the menu from Doshii.');
                        jQuery('#initialising').hide();
                        jQuery('#initialiseBtn').removeAttr('disabled','disabled');
                     }
                  })
                }
                </script>
            <?php
            break;
          case 'menu':
            $message = '';
            if (empty($token)) {
                $message = '* Doshii initialisation has not been performed.';
            }
            $locationId = $doshii_options['location_id'];
          	$doshii_product_table  = $wpdb->prefix . 'doshii_product';
            if (empty($locationId)) {
                $message = 'Doshii location ID has not been configured on the Settings tab.';
            } else {
                $table  = $wpdb->prefix . 'doshii_menu';
                $result = $wpdb->get_results("SELECT 1 FROM $doshii_product_table WHERE doshii_custom_id = 'doshii::check'");
                if ($wpdb->last_error) {
                  $message = "* Re-synchronisation is required as Doshii's menu product mapping table structure has changed.";
                }
                else {
                    $doshii_menu_sql = "SELECT doshii_filtered_menu FROM $table WHERE location_id = '$locationId'";
                    $doshiiResults = $wpdb->get_row($doshii_menu_sql) or array();
                    $filtered = empty($doshiiResults->doshii_filtered_menu) ? 'no' : $doshiiResults->doshii_filtered_menu;
                    $nowFiltered = empty($doshii_options['use_menu_management']) ? 'no' : $doshii_options['use_menu_management'];
                    if ($filtered !== $nowFiltered) {
                        $message = '* Re-synchronisation is required as Menu Management setting has changed.';
                    }
                }
            }

          	$productCount = $wpdb->get_var( "SELECT COUNT(*) FROM $doshii_product_table" );
          	$productPlural = $productCount === 1 ? 'product' : 'products';
          	$productCountMessage = empty($productCount) || $productCount === 0 ? '' : "($productCount $productPlural synchronised)";
            ?>
                <h2>
                    <?php _e('Menu Synchronisation', 'doshii'); ?>
                </h2>
                <table class="form-table">
                    <tbody>
                        <tr valign="top">
                            <th scope="row" valign="top">
                                <?php _e('Synchronise Menu', 'doshii'); ?>
                            </th>
                            <td>
                                <div>
                                    <input class="button-primary btn" type="button" value="Synchronise" id="menuSyncBtn" name="" onclick="doshii_product_sync()">
                                    <p class="menuUpdated" style="display:inline;"><?php echo $productCountMessage ?></p>
                                </div>
                                <?php printf(
                                    '<img src="%1$s" alt="progress spinner" style="display:none" id="menuProcessing"/>',
                                    plugins_url( '/assets/images/loading.gif', __FILE__ )
                                ); ?>
                            </td>
                        </tr>
                        <tr valign="top">
                            <th scope="row" valign="top"></th>
                            <td class="message"><?php echo $message ?></td>
                        </tr>
                    </tbody>
                </table>

                <script>
                function doshii_product_sync() {
                    jQuery('#menuSyncBtn').attr('disabled','disabled');
                    jQuery('#menuProcessing').show();
                    jQuery.ajax({
                     type : "GET",
                     dataType : "JSON",
                     url : '<?php echo admin_url( 'admin-ajax.php' ); ?>',
                     data : { action: "doshii_product_sync_with_woocommerce" },
                     success: function(response) {
                        jQuery('.message').html(response['message']);
                        if (response['updatedAt']) {
                            jQuery('.menuUpdated').html(`Menu last updated in Doshii on ${new Date(response['updatedAt']).toLocaleString()}`);
                        } else {
                            jQuery('.menuUpdated').html('');
                        }
                        jQuery('#menuSyncBtn').removeAttr('disabled','disabled');
                        jQuery('#menuProcessing').hide();
                     },
                     error: function(request,status,errorThrown) {
                        console.log(request);
                        console.log(status);
                        console.log(errorThrown);
                        jQuery('.menuUpdated').html('');
                        jQuery('.message').html('Failed to synchronise the menu from Doshii.');
                        jQuery('#menuProcessing').hide();
                        jQuery('#menuSyncBtn').removeAttr('disabled','disabled');
                     }
                  })
                }
                </script>
            <?php

            break;
          default:
            ?>
                <h2>
                    <?php _e('Doshii API Settings', 'doshii'); ?>
                </h2>

                <form method="post" action="options.php">
                    <?php settings_fields('doshiiapi_settings_group'); ?>
                    <table class="form-table">
                        <tbody>

                            <tr valign="top">
                                <th scope="row" valign="top">
                                    <?php _e('Doshii API Base URL *', 'doshii'); ?>
                                </th>
                                <td>
                                    <input id="doshiiapi_settings[server_url]" name="doshiiapi_settings[server_url]" type="text" class="regular-text" value="<?php if(!empty($doshii_options['server_url'])){ echo $doshii_options['server_url']; }else{ echo 'https://woocommerce-live.doshii.co'; } ?>"/>
                                </td>
                            </tr>

                            <tr valign="top">
                                <th scope="row" valign="top">
                                    <?php _e('Location ID *', 'doshii'); ?>
                                </th>
                                <td>
                                    <input id="doshiiapi_settings[location_id]" name="doshiiapi_settings[location_id]" type="text" class="regular-text" value="<?php echo $doshii_options['location_id']; ?>"/>
                                </td>
                            </tr>

                            <tr valign="top">
                                <th scope="row" valign="top">
                                    <?php _e('Shipping Fee POS ID', 'doshii'); ?>
                                </th>
                                <td>
                                    <input id="doshiiapi_settings[shipping_fee_id]" name="doshiiapi_settings[shipping_fee_id]" type="text" class="regular-text" value="<?php echo $doshii_options['shipping_fee_id']; ?>"/>
                                </td>
                            </tr>

                            <tr valign="top">
                                <th scope="row" valign="top">
                                    <?php _e('Use Menu Management', 'doshii'); ?>
                                </th>
                                <td>
                                    <input id="doshiiapi_settings[use_menu_management]" name="doshiiapi_settings[use_menu_management]" type="checkbox" value="yes" <?php checked('yes', $doshii_options['use_menu_management'], true) ?>/>
                                </td>
                            </tr>

                            <tr valign="top">
                                <th scope="row" valign="top">
                                    <?php _e('Support Bundles', 'doshii'); ?>
                                </th>
                                <td>
                                    <input id="doshiiapi_settings[support_bundles]" name="doshiiapi_settings[support_bundles]" type="checkbox" value="yes" <?php checked('yes', $doshii_options['support_bundles'], true) ?>/>
                                </td>
                            </tr>

                            <tr valign="top">
                                <th scope="row" valign="top">
                                    <?php _e('Manage Stock Availability', 'doshii'); ?>
                                </th>
                                <td>
                                    <div>
                                        <input id="doshiiapi_settings[stock_availability]" name="doshiiapi_settings[stock_availability]" type="checkbox" value="yes" <?php checked('yes', $doshii_options['stock_availability'], true) ?>/>
                                        <p style="display:inline;"> (will require re-initialisation if changed)</p>
                                    </div>
                                </td>
                            </tr>
                        </tbody>
                    </table>

                    <div class="" id="emi_field"> </div>

                    <p class="submit">
                        <input type="submit" class="button-primary" value="<?php _e('Save Options', 'mfwp_domain'); ?>" />
                    </p>
                </form>
            <?php
            break;
        endswitch; ?>
    </div>
  </div>
  <?php
}

function doshii_register_settings() {
	register_setting('doshiiapi_settings_group', 'doshiiapi_settings');
	register_setting('doshiiapi_key_group', 'doshiiapi_key');
}
add_action('admin_init', 'doshii_register_settings');
?>
