<?php
if ( ! defined( 'ABSPATH' ) ) {
    exit; // Exit if accessed directly
}
add_action( 'woocommerce_checkout_update_order_meta', 'doshii_save_custom_field_data' );

function doshii_save_custom_field_data( $order_id ) {
	if ( ! empty( $_POST['doshii_order_status'] ) ) {
        update_post_meta( $order_id, 'doshii_order_status', sanitize_text_field( $_POST['doshii_order_status'] ) );
    }
}

/**
 * Display field value on the order edit page
 */
add_action( 'woocommerce_admin_order_data_after_billing_address', 'doshii_display_admin_order_meta', 10, 1 );
function doshii_display_admin_order_meta($order) {
    global $doshii_options;
    global $doshii_api_key;

    $doshiiOrderStatus = get_post_meta( $order->get_id(), 'doshii_order_status', true );
    $doshiiOrderId = get_post_meta($order->get_id(), 'doshii_order_id', true );
    $doshiiLocationId = $doshii_options['location_id'];
    $token = $doshii_api_key['token'];
    if (empty($token) ||
        empty($doshii_options['server_url']) ||
        empty($doshiiLocationId)) {
        $doshiiOrderStatus = 'Doshii settings not configured';
    } else if (empty($doshiiOrderId)) {
        $doshiiOrderStatus = "Not synchronised with Doshii";
    } else if (empty($doshiiOrderStatus)) {
        $doshiiOrderStatus = 'Unknown';
    } else {
        $doshiiDashboard = strpos($doshii_options['server_url'], 'sandbox') !== false ?  'https://sandbox-dashboard.doshii.io' : 'https://dashboard.doshii.io';
        $doshiiDashboardUrl = $doshiiDashboard.'/venues/'.$doshiiLocationId.'/orders/'.$doshiiOrderId;
        $doshiiOrderStatus = '(#<a href="'.$doshiiDashboardUrl.'" target="_blank">'.$doshiiOrderId.'</a>) '.$doshiiOrderStatus;
    }
    echo '<p><strong>'.__('Doshii Order Status').':</strong> <br/>' . $doshiiOrderStatus . '</p>';
}

/**
 * Add a custom action to order actions select box on edit order page
 * Only added for orders that have not yet been synchronised with Doshii
 *
 * @param array $actions order actions array to display
 * @return array - updated actions
 */
function doshii_order_sync_meta_box_action( $actions ) {
    global $theorder;
    global $doshii_options;
    global $doshii_api_key;

    if (empty($doshii_api_key['token']) ||
        empty($doshii_options['server_url']) ||
        empty($doshii_options['location_id'])) {
        return $actions;
    }

    $doshiiOrderId = get_post_meta($theorder->get_id(), 'doshii_order_id', true );
    if (empty($doshiiOrderId)) {
        // Will be handled by the action defined in `woo-order-api.php`
        $actions['doshii_sync_order_action'] = __( 'Synchronise with Doshii', 'doshii' );
    }

    $doshiiOrderStatus = get_post_meta($theorder->get_id(), 'doshii_order_status', true );
    if (!empty($doshiiOrderId) && (empty($doshiiOrderStatus) || $doshiiOrderStatus != 'complete')) {
        // Will be handled by the action defined in `woo-order-status.php`
        $actions['doshii_status_order_action'] = __( 'Refresh Doshii order status', 'doshii' );
    }

    return $actions;
}
add_action( 'woocommerce_order_actions', 'doshii_order_sync_meta_box_action', 20 );
?>
