<?php
if ( ! defined( 'ABSPATH' ) ) {
    exit; // Exit if accessed directly
}

/**
 * Add the Doshii Product Map column to the exporter and the exporter column menu.
 *
 * @param array $columns
 * @return array $columns
 */
function doshii_add_export_column( $columns ) {

	// column slug => column name
	$columns['doshii_product_map'] = 'Doshii Product Map';

	return $columns;
}
add_filter( 'woocommerce_product_export_column_names', 'doshii_add_export_column' );
add_filter( 'woocommerce_product_export_product_default_columns', 'doshii_add_export_column' );

/**
 * Provide the data to be exported for one item in the column.
 *
 * @param mixed $value (default: '')
 * @param WC_Product $product
 * @return mixed $value - Should be in a format that can be output into a text file (string, numeric, etc).
 */
function doshii_export_product_map( $value, $product ) {
    $productIds = $product->get_meta( '_doshii_posId' );

    $productId = isset($productIds) ? (is_array($productIds) ? $productIds[0] : $productIds) : '';
    if (!empty($productId)) {
        $optionIds = $product->get_meta( '_doshii_option_posId' );
        $optionId = '';
        $variantId = '';
        if (isset($optionIds)) {
            if (is_array($optionIds)) {
                $optionId = !empty($optionIds[0]) ? "::{$optionIds[0]}" : '';
            } else {
                $optionId = !empty($optionIds) ? "::{$optionIds}" : '';
            }

            $variantIds = $product->get_meta( '_doshii_variant_posId' );
            if (isset($variantIds)) {
                if (is_array($variantIds)) {
                    $variantId = !empty($variantIds[0]) ? "::{$variantIds[0]}" : '';
                } else {
                    $variantId = !empty($variantIds) ? "::{$variantIds}" : '';
                }
            }
        }
        $value = "{$productId}{$optionId}{$variantId}";
    }
	return $value;
}
add_filter( 'woocommerce_product_export_product_column_doshii_product_map', 'doshii_export_product_map', 10, 2 );


/**
 * Register the 'Doshii Product Map' column in the importer.
 *
 * @param array $options
 * @return array $options
 */
function doshii_add_column_to_importer( $options ) {

	// column slug => column name
	$options['doshii_product_map'] = 'Doshii Product Map';

	return $options;
}
add_filter( 'woocommerce_csv_product_import_mapping_options', 'doshii_add_column_to_importer' );

/**
 * Add automatic mapping support for 'Doshii Product Map'.
 * This will automatically select the correct mapping for columns named 'Doshii Product Map' or 'doshii product map'.
 *
 * @param array $columns
 * @return array $columns
 */
function doshii_add_column_to_mapping_screen( $columns ) {

	// potential column name => column slug
	$columns['Doshii Product Map'] = 'doshii_product_map';
	$columns['doshii product map'] = 'doshii_product_map';

	return $columns;
}
add_filter( 'woocommerce_csv_product_import_mapping_default_columns', 'doshii_add_column_to_mapping_screen' );

/**
 * Process the data read from the CSV file.
 *
 * @param WC_Product $object - Product being imported or updated.
 * @param array $data - CSV data read for the product.
 * @return WC_Product $object
 */
function doshii_process_import( $object, $data ) {
	if (!empty($data['doshii_product_map'])) {
        $productIds = explode('::', $data['doshii_product_map'], 3);

		$productId = isset($productIds[0]) ? $productIds[0] : '';
        $object->update_meta_data( '_doshii_posId', $productId );

		$optionId = isset( $productIds[1] ) ? $productIds[1] : '';
        $object->update_meta_data( '_doshii_option_posId', $optionId );

		$variantId = isset( $productIds[2] ) ? $productIds[2] : '';
        $object->update_meta_data( '_doshii_variant_posId', $variantId );
	}

	return $object;
}
add_filter( 'woocommerce_product_import_pre_insert_product_object', 'doshii_process_import', 10, 2 );
?>
