<?php
if ( ! defined( 'ABSPATH' ) ) {
    exit; // Exit if accessed directly
}
add_action("wp_ajax_doshii_initialise_with_woocommerce", "doshii_initialise_with_woocommerce");
function doshii_initialise_with_woocommerce() {
    global $wpdb;
    global $doshii_options;
    global $doshii_api_key;

    // check user capabilities
    if ( ! current_user_can( 'manage_options' ) ) {
        return;
    }

    $locationId = $doshii_options['location_id'];
	if (empty($doshii_options['server_url']) ||
	    empty($locationId)) {
    	$sync_result = array(
            'message' => 'The Doshii settings have not been configured.'
        );
        echo wp_send_json($sync_result);
        return;
	}

    $key = sanitize_text_field($_REQUEST['key']);
	if (empty($key)) {
    	$sync_result = array(
            'message' => 'The Location Token has not been specified.'
        );
        echo wp_send_json($sync_result);
        return;
	}

	$filtered = $doshii_options['use_menu_management'];
	$filteredParam = (empty($filtered) || $filtered !== 'yes') ? '' : '?filtered=true';
	$url = $doshii_options['server_url'].'/registration'.$filteredParam;
	$headers = array(
       'Authorization' => 'Bearer '.$key.'',
       'x-doshii-location-id' => $locationId.'',
       'content-Type' => 'application/json',
       'x-doshii-writeout' => 'WooCommerce 2.5.4',
    );

    // Request webhook registration if stock availability is enabled
    $monitorStock = empty($doshii_options['stock_availability']) ? 'no' : $doshii_options['stock_availability'];
    if ($monitorStock === 'yes') {
        $webhookKey = uniqid('doshii_');
        $headers['x-doshii-webhook-key'] = $webhookKey.'';

        $adminUrl = admin_url( 'admin-post.php' );
        $headers['x-doshii-webhook-url'] = $adminUrl.'?action=doshii';
    }

    $result = wp_remote_get($url, array(
        'headers' => $headers
    ));
    if( is_wp_error( $result ) ) {
         $sync_result= array(
            'message' => 'The Doshii integration could not been initialised'
         );
         echo wp_send_json($sync_result);
        return false; // Bail early
    }

    $response = json_decode(wp_remote_retrieve_body( $result ));
	if ($response->token) {
	    $doshii_api_key['token'] = $response->token;
	    $doshii_api_key['webhook'] = $webhookKey;
	    update_option('doshiiapi_key', $doshii_api_key);

        $table  = $wpdb->prefix . 'doshii_menu';
        $delete = $wpdb->query("TRUNCATE TABLE $table");

        $table_name = $wpdb->prefix."doshii_menu";
        $filtered = empty($doshii_options['use_menu_management']) ? 'no' : $doshii_options['use_menu_management'];
        $wpdb->query("INSERT INTO $table_name (location_id, doshii_menu_version, doshii_menu_updated, doshii_filtered_menu, doshii_monitor_stock) VALUES ('$locationId', '', '', '$filtered', '$monitorStock')");

        $sync_result = array(
            'message' => 'The Doshii integration has been successfully initialised'
        );
	} else {
        $sync_result = array(
            'message' => 'No credentials obtained from Doshii.'
        );
	}
    echo wp_send_json($sync_result);
}
?>
