<?php
if ( ! defined( 'ABSPATH' ) ) {
    exit; // Exit if accessed directly
}
add_action("wp_ajax_doshii_product_sync_with_woocommerce", "doshii_product_sync_with_woocommerce");
function doshii_product_sync_with_woocommerce() {
    global $wpdb;
    global $doshii_options;
    global $doshii_api_key;

	$token = $doshii_api_key['token'];
    $locationId = $doshii_options['location_id'];

	if (empty($token) ||
	    empty($doshii_options['server_url']) ||
	    empty($locationId)) {
    	$sync_result = array(
            'message' => 'The Doshii settings have not been configured.'
        );
        echo wp_send_json($sync_result);
        return;
	}

    // Check to see if the database structure is using the latest schema, if not, upgrade
    $product_table = $wpdb->prefix . 'doshii_product';
    $result = $wpdb->get_results("SELECT 1 FROM $product_table WHERE doshii_custom_id = 'doshii::check'");
    if ($wpdb->last_error) {
        doshii_on_activation();
    }

    $table = $wpdb->prefix . 'doshii_menu';
    $doshii_menu_sql = "SELECT doshii_menu_version, doshii_menu_updated, doshii_filtered_menu FROM $table WHERE location_id = '$locationId'";
    $doshii_menu_results = $wpdb->get_row($doshii_menu_sql) or array();
    $doshii_menu_version = $doshii_menu_results->doshii_menu_version;
    $doshii_filtered_menu = empty($doshii_menu_results->doshii_filtered_menu) ? 'no' : $doshii_menu_results->doshii_filtered_menu;
    $nowFiltered = empty($doshii_options['use_menu_management']) ? 'no' : $doshii_options['use_menu_management'];

	$url = $doshii_options['server_url'].'/menu';
	if ($doshii_options['use_menu_management'] == 'yes') { $url = $url.'?filtered=true'; } // use filtered menu
	if ($doshii_menu_version && $doshii_filtered_menu == $nowFiltered) {
	    if ($doshii_options['use_menu_management'] == 'yes') { $url = $url.'&'; } else { $url = $url.'?'; }
	    $url = $url.'lastVersion='.$doshii_menu_version; // use lastVersion version
	}

    $result = wp_remote_get($url, array(
        'headers' => array(
            'Authorization' => $token.'',
            'x-doshii-location-id' => $locationId.'',
            'content-Type' => 'application/json',
            'x-doshii-writeout' => 'WooCommerce 2.5.4',
        )
    ));
    if( is_wp_error( $result ) ) {
         $sync_result= array(
            'message' => 'The Doshii menu could not be synchronised'
         );
         echo wp_send_json($sync_result);
        return false; // Bail early
    }

    switch (wp_remote_retrieve_response_code($result)) {
        case 304: # Not Modified
            $sync_result= array(
            'message' => 'The Doshii menu has not changed since last synchronisation',
            'updatedAt' => $doshii_menu_results->doshii_menu_updated
            );
            echo wp_send_json($sync_result);
            break;
    }

	$sync_result= array(
        'message' => ''
    );
    $response = json_decode(wp_remote_retrieve_body( $result ));

    $table  = $wpdb->prefix . 'doshii_menu';
    $delete = $wpdb->query("TRUNCATE TABLE $table");

    $table_name = $wpdb->prefix."doshii_menu";
    $monitorStock = empty($doshii_options['stock_availability']) ? 'no' : $doshii_options['stock_availability'];
    $wpdb->query("INSERT INTO $table_name (location_id, doshii_menu_version, doshii_menu_updated, doshii_filtered_menu, doshii_monitor_stock) VALUES ('$locationId', '$response->version', '$response->updatedAt', '$nowFiltered', '$monitorStock')");

    $delete = $wpdb->query("TRUNCATE TABLE $product_table");

    $table  = $wpdb->prefix . 'doshii_product_option';
    $delete = $wpdb->query("TRUNCATE TABLE $table");

    $table  = $wpdb->prefix . 'doshii_product_variant';
    $delete = $wpdb->query("TRUNCATE TABLE $table");

	if ($response->products) {
        foreach ( $response->products as $productData ) {
            $productName = json_encode($productData->name);
            $productPosId = $productData->posId;
            $productCustomPosId = !empty($productData->customPosId) ? $productData->customPosId : "";
            $wpdb->query("INSERT INTO $product_table (doshii_product_name, doshii_product_id, doshii_custom_id) VALUES ($productName, '$productPosId', '$productCustomPosId')");

            foreach($productData->options as $optionData) {
                $table_name = $wpdb->prefix."doshii_product_option";
                $optionName = json_encode($optionData->name);
                $optionPosId = $optionData->posId;
                $optionCustomPosId = !empty($optionData->customPosId) ? $optionData->customPosId : "";
                $wpdb->query("INSERT INTO $table_name (doshii_product_option_name, doshii_product_option_id, doshii_product_id, doshii_custom_id) VALUES ($optionName, '$optionPosId', '$productPosId', '$optionCustomPosId')");

                foreach($optionData->variants as $variantData) {
                    $table_name = $wpdb->prefix."doshii_product_variant";
                    $variantName = json_encode($variantData->name);
                    $variantPosId = $variantData->posId;
                    $variantCustomPosId = !empty($variantData->customPosId) ? $variantData->customPosId : "";
                    $wpdb->query("INSERT INTO $table_name (doshii_product_variant_name, doshii_product_variant_id, doshii_product_id, doshii_product_option_id, doshii_custom_id) VALUES ($variantName, '$variantPosId', '$productPosId', '$optionPosId', '$variantCustomPosId')");
                }
            }
        }

        $sync_result = array(
            'message' => 'The Doshii menu has been successfully synchronised with '.count($response->products).' products.',
            'updatedAt' => $response->updatedAt
        );
	} else {
        $sync_result = array(
            'message' => 'No products found in Doshii.'
        );
	}
    echo wp_send_json($sync_result);
}
?>
