<?php
if ( ! defined( 'ABSPATH' ) ) {
    exit; // Exit if accessed directly
}

add_action("admin_post_nopriv_doshii", "doshii_process_webhook_nopriv");
function doshii_process_webhook_nopriv() {
    global $doshii_options;
    global $doshii_api_key;

    $allowStockAvailability = $doshii_options['stock_availability'];
    if (empty($allowStockAvailability) || $allowStockAvailability !== 'yes') {
        status_header(422);
        exit;
    }

    $locationId = $doshii_options['location_id'];
    $webhookKey = $doshii_api_key['webhook'];
    $requestApiKey = $_SERVER['HTTP_X_DOSHII_API_KEY'];
    if (empty($locationId) || empty($requestApiKey) || empty($webhookKey) || $requestApiKey !== $webhookKey) {
        status_header(403);
        exit;
    }
    $inputJSON = file_get_contents('php://input');
    $webhookRequest = json_decode($inputJSON, TRUE); //convert JSON into array

    $posId = sanitize_text_field($webhookRequest['posId']);
    $availability = sanitize_text_field($webhookRequest['availability']);
    $requestLocationId = sanitize_text_field($webhookRequest['locationId']);
    if (empty($requestLocationId) || $locationId !== $requestLocationId) {
        status_header(401);
        exit;
    }

    if (!empty($posId) && !empty($availability)) {
        $args = array(
            'post_type' => 'product',
            'meta_key' => '_doshii_posId',
            'meta_value' => $posId,
            'meta_compare' => 'IN'
        );

        $products = wc_get_products($args);
        foreach ($products as $product) {
            $productId = $product->get_id();
            $stockStatus = $availability === 'available' ? 'instock' : 'outofstock';

            $currentStatus = get_post_meta($productId, '_stock_status', true);
            if ($currentStatus !== $stockStatus) {
                // 2. Updating the stock quantity
                update_post_meta( $productId, '_stock_status', wc_clean( $stockStatus ) );

                // 3. Updating post term relationship
                if ($stockStatus === 'instock') {
                    wp_remove_object_terms($productId, 'outofstock', 'product_visibility');
                } else {
                    wp_set_post_terms($productId, 'outofstock', 'product_visibility', true);
                }

                // And finally (optionally if needed)
                wc_delete_product_transients( $productId ); // Clear/refresh the variation cache
            }
        }
    }
}
?>
