<?php
if ( ! defined( 'ABSPATH' ) ) {
    exit; // Exit if accessed directly
}

// Create Order in Doshii Api
add_action('woocommerce_payment_complete', 'doshii_submit_order_by_id', 10, 1);
add_action('woocommerce_checkout_order_processed', 'doshii_submit_order_by_id', 10, 1);
add_action('woocommerce_order_action_doshii_sync_order_action', 'doshii_submit_order');

function doshii_submit_order( $order ) {
    global $doshii_options, $doshii_api_key, $wpdb;
    global $product,$woocommerce;
    global $product_object, $post;

    $token = $doshii_api_key['token'];
    $doshiiOrderId = get_post_meta($order->get_id(), 'doshii_order_id', true );

    // We don't need to contact Doshii if the settings are not configured or the order already has a Doshii order ID
    if (empty($token) ||
        empty($doshii_options['server_url']) ||
        empty($doshii_options['location_id']) ||
        !empty($doshiiOrderId)) {
        return;
    }

    $itemData = [];
    $transactionsData =[];
    $surcounts = [];
    $taxes = [];
    $doshiiOrderType = '';

    $orderStatus = $order->get_status();
    $paymentMethod = $order->get_payment_method();
    if ($paymentMethod == 'cod') {
        // if the order status is not pending, then we have already sent the order
        if ($orderStatus != 'pending') {
            return;
        }
        $paymentMethod = 'cash';
        array_push($transactionsData);
    } else {
        // if the order status is pending, then payment is not complete so ignore.
        if ($orderStatus == 'pending') {
            return;
        }

        // Only add the payment to Doshii if the status is processing or completed,
        // otherwise, the payment has failed and we shouldn't include it in the payload
        if ($orderStatus == 'processing' || $orderStatus == 'completed') {
            // stripe or paypal
            $paymentMethod = $paymentMethod === 'paypal' ? 'paypal' : 'credit';
            $transactionAmount = doshii_to_cents($order->get_total());
            $paymentMethodTitle = $order->get_payment_method_title();

            $yithGiftcards = $order->get_meta('_ywgc_applied_gift_cards');
            if (!empty($yithGiftcards)) {
                foreach($yithGiftcards as $key => $value) {
                    $transaction = (object)[
                       'amount' => doshii_to_cents($value),
                       'method' => 'giftcard',
                       'tip' => 0,
                       'reference' => strval($key),
                       'prepaid' => true,
                       'surcounts' => []
                    ];
                    array_push($transactionsData, $transaction);
                }
            }
            if ($transactionAmount > 0) {
                $transaction = (object)[
                   'amount' => $transactionAmount,
                   'method' => $paymentMethod,
                   'tip' => 0,
                   'prepaid' => true,
                   'surcounts' => []
                ];
                $transactionId = $order->get_transaction_id();
                if (!empty($transactionId)) {
                    $transaction->reference = $paymentMethodTitle.': '.$transactionId;
                    $transaction->invoice = strval($transactionId);
                }
                array_push($transactionsData, $transaction);
            }
        }
    }

    $order_id = $order->get_id();
    $date_created = $order->get_date_created();

    if (empty($order->get_shipping_first_name())) {
        $first_name = $order->get_billing_first_name();
        $last_name = $order->get_billing_last_name();
        $address1 = $order->get_billing_address_1();
        $address2 = $order->get_billing_address_2();
        $city = $order->get_billing_city();
        $state = empty($order->get_billing_state()) ? doshii_state_from_postcode($order->get_billing_postcode()) : $order->get_billing_state();
        $postalCode = $order->get_billing_postcode();
        $country =  $order->get_billing_country();
        $doshiiOrderType = 'pickup';
     } else {
        $first_name = $order->get_shipping_first_name();
        $last_name = $order->get_shipping_last_name();
        $address1 = $order->get_shipping_address_1();
        $address2 = $order->get_shipping_address_2();
        $city = $order->get_shipping_city();
        $state = empty($order->get_shipping_state()) ? doshii_state_from_postcode($order->get_shipping_postcode()) : $order->get_shipping_state();
        $postalCode = $order->get_shipping_postcode();
        $country =  $order->get_shipping_country();
        $doshiiOrderType = 'delivery';
    }
    $email = $order->get_billing_email();
    $phone = $order->get_billing_phone();

    $shippingTotal = doshii_to_cents($order->get_shipping_total() + $order->get_shipping_tax());
    if ($shippingTotal && $shippingTotal > 0) {
        $shippingFeePosId = $doshii_options['shipping_fee_id'];
        if (empty($shippingFeePosId)) {
            array_push($surcounts, (object)[
                'name' => 'Shipping Fee',
                'amount' => $shippingTotal,
                'type' => 'absolute',
                'value' => $shippingTotal
            ]);
        } else {
            array_push($surcounts, (object)[
                'posId' => $shippingFeePosId,
                'name' => 'Shipping Fee',
                'amount' => $shippingTotal,
                'type' => 'absolute',
                'value' => $shippingTotal
            ]);
        }
    }

    $taxTotal = doshii_to_cents($order->get_cart_tax());
    array_push($taxes, (object)[
        'name' => 'Tax',
        'amount' => $taxTotal,
        'type' => 'absolute',
        'taxType' => $order->get_prices_include_tax() ? 'inclusive' : 'exclusive',
        'value' => $taxTotal
    ]);

    $fullname = $first_name.' '.$last_name;
    $externalOrderRef = $order_id;

    $woocommerceItems = [];
    $parentBundleItems = array();
    $doshii_product_tbl  = $wpdb->prefix . 'doshii_product';
    foreach ( $order->get_items() as $item_id => $item ) {
        array_push($woocommerceItems, $item->get_data());
        $product_id = $item->get_product_id();
        $variation_id = $item->get_variation_id();
        $product = $item->get_product();
        $product_price = $product->get_price();

        $name = $item->get_name();
        $quantity = $item->get_quantity();
        $subtotal = $order->get_line_total($item, $order->get_prices_include_tax(), true);
        $productType = 'single';

        $productPrice = doshii_to_cents($product_price);
        $centAmount = doshii_to_cents($subtotal);

        $mappedProductId = get_post_meta($product_id, '_doshii_posId');
        $mappedProductId = count($mappedProductId) > 0 ? $mappedProductId[0] : '';
        $mappedProductId = is_array($mappedProductId) ? $mappedProductId[0] : $mappedProductId;
        $mappedOptionId = empty($mappedProductId) ? [] : get_post_meta($product_id, '_doshii_option_posId');
        $mappedOptionId = count($mappedOptionId) > 0 ? $mappedOptionId[0] : '';
        $mappedOptionId = is_array($mappedOptionId) ? $mappedOptionId[0] : $mappedOptionId;
        $product_customPosId = '';

        // Check if we have mapped the WooCommerce product to a customPosId (using the WooCommerce product ID/slug)
        if (empty($mappedProductId)) {
            $doshii_product_sql = "SELECT * FROM $doshii_product_tbl where doshii_custom_id = '$product_id'";
            $doshii_product_record = $wpdb->get_row($doshii_product_sql);
            $product_posId = $doshii_product_record->doshii_product_id;
            if (!empty($product_posId)) {
                $mappedProductId = $product_posId;
            }
        } else {
            // Check if there is a doshii:virtual custom POS ID
            $doshii_product_sql = "SELECT * FROM $doshii_product_tbl where doshii_product_id = '$mappedProductId'";
            $doshii_product_record = $wpdb->get_row($doshii_product_sql);
            $product_customPosId = $doshii_product_record->doshii_custom_id;
        }

        $doshiiOptions = [];
        if (!empty($mappedOptionId)) {
            $mappedVariantId = get_post_meta($product_id, '_doshii_variant_posId');
            $mappedVariantId = count($mappedVariantId) > 0 ? $mappedVariantId[0] : '';
            $mappedVariantId = is_array($mappedVariantId) ? $mappedVariantId[0] : $mappedVariantId;

            if (!empty($mappedVariantId)) {
                $doshii_product_option_tbl  = $wpdb->prefix . 'doshii_product_option';
                $doshii_option_sql = "SELECT * FROM $doshii_product_option_tbl where doshii_product_id = '$mappedProductId' AND doshii_product_option_id='$mappedOptionId'";
                $doshii_option_record = $wpdb->get_row($doshii_option_sql);

                $option_posId = $doshii_option_record->doshii_product_option_id;
                $option_customPosId = $doshii_option_record->doshii_custom_id;
                $option_name = $doshii_option_record->doshii_product_option_name;
                if (empty($option_name)) {
                    $option_name = 'Option';
                }

                $doshii_product_variant_tbl  = $wpdb->prefix . 'doshii_product_variant';
                $doshii_variant_sql = "SELECT * FROM $doshii_product_variant_tbl where doshii_product_id = '$mappedProductId' AND doshii_product_option_id='$mappedOptionId' AND doshii_product_variant_id='$mappedVariantId'";
                $doshii_variant_record = $wpdb->get_row($doshii_variant_sql);
                $variant_customPosId = $doshii_variant_record->doshii_custom_id;

                $variant_name = $doshii_variant_record->doshii_product_variant_name;
                if (empty($variant_name)) {
                    $variant_name = 'Modifier';
                }

                if (empty($option_posId)) {
                    $doshiiOptions = [
                        (object)[
                            'name' => $option_name,
                            'variants' => [(object)[
                                'posId' => !empty($variant_customPosId) && strpos($variant_customPosId, "doshii:") === 0 ? $variant_customPosId : $mappedVariantId,
                                'name' => $variant_name,
                                'price' => 0
                            ]]
                        ]
                    ];
                } else {
                    $doshiiOptions = [
                        (object)[
                            'posId' => $option_posId,
                            'name' => $option_name,
                            'variants' => [(object)[
                                'posId' => !empty($variant_customPosId) && strpos($variant_customPosId, "doshii:") === 0 ? $variant_customPosId : $mappedVariantId,
                                'name' => $variant_name,
                                'price' => 0
                            ]]
                        ]
                    ];
                }
            }
        }

        $bundledItems = [];
        $parentBundle = '';
        $bundleCartKey = '';
        $metaDataKeyValues = array();
        $ppomPriceAddLabelPrefix = "<span class='ppom-option-label-price'>[+&#36;";
        $ppomPriceRemoveLabelPrefix = "<span class='ppom-option-label-price'>[-&#36;";
        $doshii_product_option_tbl  = $wpdb->prefix . 'doshii_product_option';
        $doshii_product_variant_tbl  = $wpdb->prefix . 'doshii_product_variant';
        $item_meta_data = $item->get_meta_data();
        if (!empty($item_meta_data)) {
            foreach ( $item_meta_data as $meta ) {
                // If the metadata is a product attribute then process item_meta_data as an option/variant
                if (!empty($meta->key) && substr($meta->key, 0, 3) === 'pa_') {
                    $metaOptionId = substr($meta->key, 3); // strip the 'pa_' from the start

                    if (!empty($metaOptionId) && !empty($meta->value)) {
                        $doshii_option_sql = "SELECT * FROM $doshii_product_option_tbl where doshii_product_id = '$mappedProductId' AND doshii_product_option_id='$metaOptionId'";
                        $doshii_option_record = $wpdb->get_row($doshii_option_sql);

                        $option_posId = $doshii_option_record->doshii_product_option_id;
                        $option_name = $doshii_option_record->doshii_product_option_name;
                        if (empty($option_name)) {
                            $doshii_custom_option_sql = "SELECT * FROM $doshii_product_option_tbl where doshii_product_id = '$mappedProductId' AND doshii_custom_id='$metaOptionId'";
                            $doshii_custom_option_record = $wpdb->get_row($doshii_custom_option_sql);

                            $option_posId = $doshii_custom_option_record->doshii_product_option_id;
                            $option_name = $doshii_custom_option_record->doshii_product_option_name;
                            if (empty($option_name)) {
                                $option_name = $metaOptionId;
                            }
                        }

                        $variant_name = $meta->value;
                        $variant_posId = '';
                        if (!empty($option_posId)) {
                            $metaVariantId = $meta->value;
                            $doshii_variant_sql = "SELECT * FROM $doshii_product_variant_tbl where doshii_product_id = '$mappedProductId' AND doshii_product_option_id='$option_posId' AND doshii_product_variant_id='$metaVariantId'";
                            $doshii_variant_record = $wpdb->get_row($doshii_variant_sql);

                            $variant_posId = $doshii_variant_record->doshii_product_variant_id;
                            $variant_name = $doshii_variant_record->doshii_product_variant_name;
                            $variant_customPosId = $doshii_variant_record->doshii_custom_id;
                            if (empty($variant_name)) {
                                $doshii_custom_variant_sql = "SELECT * FROM $doshii_product_variant_tbl where doshii_product_id = '$mappedProductId' AND doshii_product_option_id='$option_posId' AND doshii_custom_id='$metaVariantId'";
                                $doshii_custom_variant_record = $wpdb->get_row($doshii_custom_variant_sql);

                                $variant_posId = $doshii_custom_variant_record->doshii_product_variant_id;
                                $variant_name = $doshii_custom_variant_record->doshii_product_variant_name;
                                $variant_customPosId = $doshii_custom_variant_record->doshii_custom_id;
                                if (empty($variant_name)) {
                                    $variant_name = $meta->value;
                                }
                            }
                        }

                        $variants = [];
                        if (empty($variant_posId)) {
                            array_push($variants, (object)[
                                'name' => $variant_name,
                                'price' => 0
                            ]);
                        } else {
                            array_push($variants, (object)[
                                'posId' => !empty($variant_customPosId) && strpos($variant_customPosId, "doshii:") === 0 ? $variant_customPosId : $variant_posId,
                                'name' => $variant_name,
                                'price' => 0
                            ]);
                        }

                        if (empty($option_posId)) {
                            array_push($doshiiOptions, (object)[
                                'name' => $option_name,
                                'variants' => $variants
                            ]);
                        } else {
                            array_push($doshiiOptions, (object)[
                                'posId' => $option_posId,
                                'name' => $option_name,
                                'variants' => $variants
                            ]);
                        }
                    }
                }
                // If the metadata is a PPOM option then process item_meta_data as an option/variant
                else if (!empty($meta->key) && substr($meta->key, 0, 12) === '_ppom_fields') {
                    $metaValue = $meta->value;
                    $ppomFields = $metaValue["fields"];
                    $metaPpoms = json_decode($metaValue["ppom_option_price"]);

                    foreach ( $ppomFields as $optionKey => $optionValue ) {
                        // ppom fields have id property, then all other properties are the option groups
                        // "fields": {
                        //    "id":"2",
                        //    "000011":["Coleslaw","Cheddar"],
                        //    "000024":["No Mustard","No Lettuce"],
                        //    "test_text": "Helpful text",
                        //    "test_date": "2020-10-16",
                        //    "test_number": "76543",
                        //    "test_select": "Stuff",
                        //    "test_radio": "Radio 2"
                        // },
                        // An additional meta property is included if the option has a price (included.price $0)
                        // "ppom_option_price": "[
                        //    {
                        //       "label":"Add-Ons Coleslaw",
                        //       "price":"0.70",
                        //       "apply":"variable",
                        //       "product_title":"Schnitzel Roll",
                        //       "taxable":"",
                        //       "without_tax":"",
                        //       "option_id":"000022",
                        //       "data_name":"000011"
                        //    },
                        // ]"
                        if ($optionKey !== 'id') {
                            $metaOptionId = $optionKey;

                            $doshii_option_sql = "SELECT * FROM $doshii_product_option_tbl where doshii_product_id = '$mappedProductId' AND doshii_product_option_id='$metaOptionId'";
                            $doshii_option_record = $wpdb->get_row($doshii_option_sql);

                            $option_posId = $doshii_option_record->doshii_product_option_id;
                            $option_name = $doshii_option_record->doshii_product_option_name;
                            if (empty($option_name)) {
                                $doshii_custom_option_sql = "SELECT * FROM $doshii_product_option_tbl where doshii_product_id = '$mappedProductId' AND doshii_custom_id='$metaOptionId'";
                                $doshii_custom_option_record = $wpdb->get_row($doshii_custom_option_sql);

                                $option_posId = $doshii_custom_option_record->doshii_product_option_id;
                                $option_name = $doshii_custom_option_record->doshii_product_option_name;
                                if (empty($option_name)) {
                                    $option_name = $metaOptionId;
                                }
                            }

                            $variants = [];

                            // Convert to an array if it isn't already
                            if (!is_array($optionValue)) {
                                $valueArray = array();
                                if (!empty($optionValue)) {
                                    $valueArray[] = $optionValue;
                                }
                                $optionValue = $valueArray;
                            }
                            foreach ( $optionValue as $metaVariantName ) {
                                $variant_name = $metaVariantName;
                                $variantHasPrice = false;
                                $variantPrice = 0;
                                if (empty($metaPpoms)) {
                                    // The PPOM field might have a price in the label, so will check for that, just in case
                                    // e.g. "Coleslaw <span class='ppom-option-label-price'>[+&#36;0.70]</span>"
                                    $variantLabel = $metaDataKeyValues[$optionKey];
                                    if (!empty($variantLabel)) {
                                        $startIndex = strpos($variantLabel, $metaVariantName.' '.$ppomPriceAddLabelPrefix);
                                        $startLength = strlen($metaVariantName.' '.$ppomPriceAddLabelPrefix);
                                        $priceMultiplier = 1; // assume is positive price adjustment
                                        if ($startIndex === false) {
                                            $startIndex = strpos($variantLabel, $metaVariantName.' '.$ppomPriceRemoveLabelPrefix); // check if negative
                                            $startLength = strlen($metaVariantName.' '.$ppomPriceRemoveLabelPrefix);
                                            $priceMultiplier = -1; // mark as negative
                                        }
                                        if ($startIndex !== false) {
                                            $startIndex = $startIndex + $startLength;
                                            $endIndex = $startIndex ? strpos($variantLabel, "]</span>", $startIndex) : 0;
                                            $labelPrice = substr($variantLabel, $startIndex, $endIndex);
                                            if (!empty($labelPrice)) {
                                                $variantPrice = doshii_to_cents($labelPrice) * $priceMultiplier;
                                            }
                                        }
                                    }
                                } else {
                                    foreach ( $metaPpoms as $ppomOptionVariant ) {
                                        $variantLength = strlen($metaVariantName);
                                        if ($ppomOptionVariant->data_name === $optionKey && $variantLength > 0 && !empty($ppomOptionVariant->label) && substr($ppomOptionVariant->label, -$variantLength) === $metaVariantName) {
                                            $variantHasPrice = true;
                                            $metaVariantId = $ppomOptionVariant->option_id;
                                            $doshii_variant_sql = "SELECT * FROM $doshii_product_variant_tbl where doshii_product_id = '$mappedProductId' AND doshii_product_option_id='$option_posId' AND doshii_product_variant_id='$metaVariantId'";
                                            $doshii_variant_record = $wpdb->get_row($doshii_variant_sql);

                                            $variant_posId = $doshii_variant_record->doshii_product_variant_id;
                                            $variant_name = $doshii_variant_record->doshii_product_variant_name;
                                            $variant_customPosId = $doshii_variant_record->doshii_custom_id;
                                            if (empty($variant_name)) {
                                                $doshii_custom_variant_sql = "SELECT * FROM $doshii_product_variant_tbl where doshii_product_id = '$mappedProductId' AND doshii_product_option_id='$option_posId' AND doshii_custom_id='$metaVariantId'";
                                                $doshii_custom_variant_record = $wpdb->get_row($doshii_custom_variant_sql);

                                                $variant_posId = $doshii_custom_variant_record->doshii_product_variant_id;
                                                $variant_name = $doshii_custom_variant_record->doshii_product_variant_name;
                                                $variant_customPosId = $doshii_custom_variant_record->doshii_custom_id;
                                                if (empty($variant_name)) {
                                                    $variant_name = empty($ppomOptionVariant->label) ? $metaVariantName : $ppomOptionVariant->label;
                                                }
                                            }

                                            if (empty($variant_posId)) {
                                                array_push($variants, (object)[
                                                    'name' => $variant_name,
                                                    'price' => doshii_to_cents($ppomOptionVariant->price)
                                                ]);
                                            } else {
                                                array_push($variants, (object)[
                                                    'posId' => !empty($variant_customPosId) && strpos($variant_customPosId, "doshii:") === 0 ? $variant_customPosId : $variant_posId,
                                                    'name' => $variant_name,
                                                    'price' => doshii_to_cents($ppomOptionVariant->price)
                                                ]);
                                            }
                                        }
                                    }
                                }

                                if (!$variantHasPrice) {
                                    array_push($variants, (object)[
                                        'name' => $variant_name,
                                        'price' => $variantPrice
                                    ]);
                                }
                            }

                            // Only add the option if there are variants in the list
                            if (!empty($variants)) {
                                if (empty($option_posId)) {
                                    array_push($doshiiOptions, (object)[
                                        'name' => $option_name,
                                        'variants' => $variants
                                    ]);
                                } else {
                                    array_push($doshiiOptions, (object)[
                                        'posId' => $option_posId,
                                        'name' => $option_name,
                                        'variants' => $variants
                                    ]);
                                }
                            }
                        }
                    }
                }
                // If the metadata is a bundle then we need to set the product type to 'bundle'
                else if (!empty($meta->key) && substr($meta->key, 0, 18) === '_bundle_group_mode') {
                    $productType = 'bundle';
                }
                // If the metadata is for the parent's bundled items, then we store them
                else if (!empty($meta->key) && substr($meta->key, 0, 14) === '_bundled_items') {
                    $bundledItems = $meta->value;
                }
                // If the metadata indicates this is a bundled item, then we capture the parent item
                else if (!empty($meta->key) && substr($meta->key, 0, 11) === '_bundled_by') {
                    $parentBundle = $meta->value;
                }
                // If the metadata indicates this is part of a bundle, then we capture the bundle item key
                else if (!empty($meta->key) && substr($meta->key, 0, 16) === '_bundle_cart_key') {
                    $bundleCartKey = $meta->value;
                }
                // Capture any key/values in case they are required in the PPOM processing
                else {
                    $metaDataKeyValues[$meta->key] = $meta->value;

                    // Handle custom attributes
                    $customAttributeLabel = wc_attribute_label($meta->key, $product);
                    if (!empty($customAttributeLabel) && $customAttributeLabel !== $meta->key) {
                        $customOptionId = $meta->key;
                        $customVariantId = $meta->value;

                        if (!empty($customVariantId)) {
                            $doshii_option_sql = "SELECT * FROM $doshii_product_option_tbl where doshii_product_id = '$mappedProductId' AND doshii_product_option_id='$customOptionId'";
                            $doshii_option_record = $wpdb->get_row($doshii_option_sql);

                            $option_posId = $doshii_option_record->doshii_product_option_id;
                            $option_name = $doshii_option_record->doshii_product_option_name;
                            if (empty($option_name)) {
                                $doshii_custom_option_sql = "SELECT * FROM $doshii_product_option_tbl where doshii_product_id = '$mappedProductId' AND doshii_custom_id='$customOptionId'";
                                $doshii_custom_option_record = $wpdb->get_row($doshii_custom_option_sql);

                                $option_posId = $doshii_custom_option_record->doshii_product_option_id;
                                $option_name = $doshii_custom_option_record->doshii_product_option_name;
                                if (empty($option_name)) {
                                    $option_name = $customAttributeLabel;
                                }
                            }

                            $variant_name = $customVariantId;
                            $variant_posId = '';
                            if (!empty($option_posId)) {
                                $doshii_variant_sql = "SELECT * FROM $doshii_product_variant_tbl where doshii_product_id = '$mappedProductId' AND doshii_product_option_id='$option_posId' AND doshii_product_variant_id='$customVariantId'";
                                $doshii_variant_record = $wpdb->get_row($doshii_variant_sql);

                                $variant_posId = $doshii_variant_record->doshii_product_variant_id;
                                $variant_name = $doshii_variant_record->doshii_product_variant_name;
                                $variant_customPosId = $doshii_variant_record->doshii_custom_id;
                                if (empty($variant_name)) {
                                    $doshii_custom_variant_sql = "SELECT * FROM $doshii_product_variant_tbl where doshii_product_id = '$mappedProductId' AND doshii_product_option_id='$option_posId' AND doshii_custom_id='$customVariantId'";
                                    $doshii_custom_variant_record = $wpdb->get_row($doshii_custom_variant_sql);

                                    $variant_posId = $doshii_custom_variant_record->doshii_product_variant_id;
                                    $variant_name = $doshii_custom_variant_record->doshii_product_variant_name;
                                    $variant_customPosId = $doshii_custom_variant_record->doshii_custom_id;
                                    if (empty($variant_name)) {
                                        $variant_name = $customVariantId;
                                    }
                                }
                            }

                            $variants = [];
                            if (empty($variant_posId)) {
                                array_push($variants, (object)[
                                    'name' => $variant_name,
                                    'price' => 0
                                ]);
                            } else {
                                array_push($variants, (object)[
                                    'posId' => !empty($variant_customPosId) && strpos($variant_customPosId, "doshii:") === 0 ? $variant_customPosId : $variant_posId,
                                    'name' => $variant_name,
                                    'price' => 0
                                ]);
                            }

                            if (empty($option_posId)) {
                                array_push($doshiiOptions, (object)[
                                    'name' => $option_name,
                                    'variants' => $variants
                                ]);
                            } else {
                                array_push($doshiiOptions, (object)[
                                    'posId' => $option_posId,
                                    'name' => $option_name,
                                    'variants' => $variants
                                ]);
                            }
                        }
                    }
                }
            }

            if ($productType === 'bundle' && empty($bundledItems)) {
                $productType = 'single';
            }
        }

        $doshiiItem = (object)[
            'name' => $name,
            'quantity' => $quantity,
            'unitPrice' => $productPrice, // default here, but re-calculated later
            'options' => $doshiiOptions
        ];
        if (!empty($mappedProductId)) {
            $doshiiItem->posId = !empty($product_customPosId) && strpos($product_customPosId, "doshii:") === 0 ? $product_customPosId : $mappedProductId;
        }
        if ($doshii_options['support_bundles'] === 'yes' && !empty($parentBundle)) {
            $doshiiBundleItem = $parentBundleItems[$parentBundle];
            if (!is_null($doshiiBundleItem)) {
                $doshiiItem->unitPrice = 0; // set all included items to $0
                array_push($doshiiBundleItem->includedItems, $doshiiItem);
            }
        } else {
            if ($doshii_options['support_bundles'] === 'yes' && $productType === 'bundle') {
                $doshiiItem->type = $productType;
                $doshiiItem->includedItems = [];

                if (!empty($bundleCartKey)) {
                    $parentBundleItems[$bundleCartKey] = $doshiiItem;
                }
            }
            $doshiiItem->totalBeforeSurcounts = $centAmount;
            $doshiiItem->totalAfterSurcounts = $centAmount;
            $doshiiItem->surcounts = [];

            // Safeguard on item price, in case there are 3rd party plugins altering the price
            // Calculate backwards to get the unit price
            $optionVariantPrice = 0;
            foreach($doshiiOptions as $option) {
                foreach($option->variants as $variant) {
                    $optionVariantPrice += $variant->price;
                }
            }
            $doshiiItem->unitPrice = ($centAmount / $quantity) - $optionVariantPrice;
            array_push($itemData, $doshiiItem);
        }
    }

    $doshiiOrder = (object)[
       'externalOrderRef' => strval($externalOrderRef),
       'type' => $doshiiOrderType,
       'items' => $itemData,
       'surcounts' => $surcounts,
       'taxes' => $taxes
    ];

    // We currently only support the Order Delivery Date for WooCommerce (Lite/Pro)
    // https://www.tychesoftwares.com/docs/docs/order-delivery-date-for-woocommerce-lite/
    $deliveryDate = $order->get_meta('_orddd_lite_timestamp') or $order->get_meta('_orddd_pro_timestamp');
    if (!empty($deliveryDate) && intval($deliveryDate) > time()) {
        // The delivery date is in epoch seconds, however, the date selected by user is local timezone
        // - doshii_submit_order_by_id we need to create as WP timezone, then convert to UTC
        $requiredAt = date_format((new DateTime(date('Y-m-d H:i:s', $deliveryDate), wp_timezone()))->setTimezone(new DateTimeZone('UTC')), 'c');

        $doshiiOrder->requiredAt = $requiredAt;
    }

    $consumerAddress = (object)[
        'line1' => empty($address1) ? '-' : $address1,
        'city' => empty($city) ? '-' : $city,
        'state' => empty($state) ? '-' : $state,
        'postalCode' => empty($postalCode) ? '-' : $postalCode,
        'country' => empty($country) ? 'AU' : $country
    ];
    if (!empty($address2)) {
        $consumerAddress->line2 = $address2;
    }

    $notes = $order->get_customer_note();
    if (!empty($notes)) {
        $consumerAddress->notes = $notes;
        $doshiiOrder->notes = strlen($notes) > 255 ? substr($notes, 255) : $notes;
    }

    $woocommerceOrder = $order->get_data();
    $woocommerceOrder['order_items'] = $woocommerceItems;
    $doshiiOrderPayload = (object) [
        'order' => $doshiiOrder,
        'consumer' => (object)[
            'name' => $fullname,
            'email' => empty($email) ? null : $email,
            'phone' => empty($phone) ? '-' : $phone,
            'address' => $consumerAddress
        ],
        'transactions' => $transactionsData,
        'woocommerce' => $woocommerceOrder
    ];

    // If we don't currently have a Doshii order ID associated, then we should push the order to Doshii
    $doshiiOrder_id = get_post_meta($order_id, 'doshii_order_id', true );
    if (empty($doshiiOrder_id)) {
        $result = wp_remote_post($doshii_options['server_url'].'/orders', array(
            'headers' => array(
                'Authorization' => $token.'',
                'x-doshii-location-id' => $doshii_options['location_id'].'',
                'content-Type' => 'application/json',
                'accept' => 'application/json',
                'Accept-encoding' => 'gzip',
                'x-doshii-writeout' => 'WooCommerce 2.5.4',
            ),
            'body' => json_encode($doshiiOrderPayload)
        ));

        if (!is_wp_error($result)) {
            $response = json_decode(wp_remote_retrieve_body( $result ));
            if ($response->id) {
                add_post_meta( $order_id, 'doshii_order_id', $response->id, true );
                update_post_meta( $order_id, 'doshii_order_status', $response->status );
                delete_post_meta( $order_id, 'doshii_sync_error');
            } else {
                add_post_meta( $order_id, 'doshii_sync_error', json_encode($response), true );
                error_log('Failed to submit order to Doshii: '.json_encode($response));
            }
        }
    }
}

function doshii_submit_order_by_id( $order_id ) {
    $order = wc_get_order( $order_id );

    if ($order) {
        doshii_submit_order($order);
    }
}

function doshii_to_cents($value) {
    return empty($value)
        ? 0
        : intval(strval(floatval(preg_replace("/[^0-9.\-]/", "", $value)) * 100));
}

function doshii_state_from_postcode($postcode) {
    if (empty($postcode)) {
        return '-';
    }

    $code = intval($postcode);
    if (($code >= 2000 && $code <= 2599) || ($code >= 2619 && $code <= 2899) || ($code >= 2921 && $code <= 2999)) {
        return 'NSW';
    }
    if (($code >= 2600 && $code <= 2618) || ($code >= 2900 && $code <= 2920)) {
        return 'ACT';
    }
    if (($code >= 3000 && $code <= 3999) || ($code >= 8000 && $code <= 8999)) {
        return 'VIC';
    }
    if (($code >= 4000 && $code <= 4999) || ($code >= 9000 && $code <= 9999)) {
        return 'QLD';
    }
    if (($code >= 5000 && $code <= 5999)) {
        return 'SA';
    }
    if (($code >= 6000 && $code <= 6999)) {
        return 'WA';
    }
    if (($code >= 7000 && $code <= 7999)) {
        return 'TAS';
    }
    if (($code >= 800 && $code <= 999)) {
        return 'NT';
    }

    return '-';
}
?>
