<?php
if ( ! defined( 'ABSPATH' ) ) {
    exit; // Exit if accessed directly
}
/**
 * Update the order meta with field value
 */
add_action( 'woocommerce_admin_order_data_after_order_details', 'refresh_doshii_pending_order' );
add_action( 'woocommerce_order_action_doshii_status_order_action', 'refresh_doshii_order_status' );

function refresh_doshii_pending_order( $order ) {
    $doshiiOrder_id = get_post_meta($order->get_id(), 'doshii_order_id', true );
    $doshiiOrderStatus = get_post_meta($order->get_id(), 'doshii_order_status', true );

    if (!empty($doshiiOrder_id) && $doshiiOrderStatus == 'pending') {
        refresh_doshii_order_status($order);
    }
}

function refresh_doshii_order_status( $order ) {
    global $doshii_options, $doshii_api_key, $wpdb;

    $token = $doshii_api_key['token'];

    if (empty($token) ||
        empty($doshii_options['server_url']) ||
        empty($doshii_options['location_id'])) {
        return; // We don't need to contact Doshii if the settings are not configured
    }

    $doshiiOrder_id = get_post_meta($order->get_id(), 'doshii_order_id', true );
    $doshiiOrderStatus = get_post_meta( $order->get_id(), 'doshii_order_status', true );

    if (!empty($doshiiOrder_id) && $doshiiOrderStatus != 'complete') {
        $url = $doshii_options['server_url'].'/orders/'.$doshiiOrder_id;
        $result = wp_remote_get($url, array(
            'headers' => array(
                'Authorization' => $token.'',
                'x-doshii-location-id' => $doshii_options['location_id'].'',
                'content-Type' => 'application/json',
                'x-doshii-writeout' => 'WooCommerce 2.5.4',
            )
        ));
        if(!is_wp_error( $result ) ) {
            $response = json_decode(wp_remote_retrieve_body( $result ));
            if (!empty($response)) {
                if ($doshiiOrder_id == $response->id) {
                    update_post_meta( $order->get_id(), 'doshii_order_status', $response->status );
                }
            }
        }
    }
}

/**
 * Adds 'Doshii Order' column header to 'Orders' page immediately after 'Status' column.
 *
 * @param string[] $columns
 * @return string[] $new_columns
 */
function doshii_add_doshii_order_column_header( $columns ) {

    $new_columns = array();

    foreach ( $columns as $column_name => $column_info ) {

        $new_columns[ $column_name ] = $column_info;

        if ( 'order_status' === $column_name ) {
            $new_columns['order_doshii'] = __( 'Doshii', 'doshii' );
        }
    }

    return $new_columns;
}
add_filter( 'manage_edit-shop_order_columns', 'doshii_add_doshii_order_column_header', 20 );

/**
 * Adds 'Doshii Order' column content to 'Orders' page immediately after 'Status' column.
 *
 * @param string[] $column name of column being displayed
 */
function doshii_add_doshii_order_column_content( $column ) {
    global $post;
    global $doshii_options;
    global $doshii_api_key;

    if ( 'order_doshii' === $column ) {
        $order = wc_get_order( $post->ID );
        $doshiiOrderStatus = get_post_meta( $order->get_id(), 'doshii_order_status', true );
        $doshiiOrderId = get_post_meta($order->get_id(), 'doshii_order_id', true );
        $doshiiLocationId = $doshii_options['location_id'];
        if (empty($doshii_api_key['token']) ||
            empty($doshii_options['server_url']) ||
            empty($doshiiLocationId)) {
            $doshiiOrderStatus = 'N/A';
        } else if (empty($doshiiOrderId)) {
            $doshiiOrderStatus = "Not Sync'ed";
        } else if (empty($doshiiOrderStatus)) {
            $doshiiOrderStatus = 'Unknown';
        } else {
            $doshiiDashboard = strpos($doshii_options['server_url'], 'sandbox') !== false ?  'https://sandbox-dashboard.doshii.io' : 'https://dashboard.doshii.io';
            $doshiiDashboardUrl = $doshiiDashboard.'/venues/'.$doshiiLocationId.'/orders/'.$doshiiOrderId;
            $doshiiOrderStatus = '(#<a href="'.$doshiiDashboardUrl.'" target="_blank">'.$doshiiOrderId.'</a>) '.$doshiiOrderStatus;
        }
        echo $doshiiOrderStatus;
    }
}
add_action( 'manage_shop_order_posts_custom_column', 'doshii_add_doshii_order_column_content' );
?>
